# bash base library version 0.2.4
# (C) Copyright  Heinz Deinhart 2015-2018
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# This is a bash library, it has to be sourced not called.
#
# bash-base provides a number of functions and a base setup of settings
# to support clean coding style in bash.
#
# Internal global veriables are prefixed with _BASH_BASE_, internal
# function with _bash_base_

[ "$0" != "$BASH_SOURCE" ] || {
	echo "$0: this script must be sourced, not called" >&2
	exit 1
}

# log <message> [..]
# logs message to stderr and if define to file name in $LOG_FILE
function log() {
    echo "$(date +'%Y%m%d %H:%M.%S') ${0##*/}[${BASHPID}]: $*" >&2 || true
    if [ "${LOG_FILE+isset}" == "isset" ]
    then
	echo "$(date +'%Y%m%d %H:%M.%S') ${0##*/}[${BASHPID}]: $*" \
	    >> "$LOG_FILE" || true
    fi
}

# log_debug <level> <message> [..]
# logs message if $LOG_LEVEL is set and it is level <= $LOG_LEVEL
function log_debug() {
    (( $# > 1 )) || fatal "log_debug call with $# instead of 2+ args"
    local LVL="$1"
    shift

    if [ "${LOG_LEVEL+isset}" == "isset" ]
    then
	if (( "$LOG_LEVEL" < "$1" ))
	then
	    log "debug<$LVL$> $*"
	fi
    fi
}

# fatal <message> [..]
# end program execution after logging message prepended with 'fatal error'
function fatal() {
    bb_traps_disable
    log "fatal error: $*, stack trace follows:"
    _bash_base_log_stacktrace >&2
    exit 1
}

# bash_stacktrace
function _bash_base_log_stacktrace() {
    local i
    for ((i=1; i < (${#FUNCNAME[@]} - 1); i++))
    do
        log "${1-}${BASH_SOURCE[$i+1]}:${BASH_LINENO[$i]} ${FUNCNAME[$i]}"
    done
}

function _bash_base_err_exit_trap() {
    local status=$?
    bb_traps_disable
    log "unexpected exit/error near '$BASH_COMMAND', status=$status, trap=$1, stack trace follows:"
    _bash_base_log_stacktrace
}

# bb_traps_setup
# sets up traps for ERR and EXIT to handle unexpected errors and exits
function bb_traps_setup() {
    trap '_bash_base_err_exit_trap "ERR"' ERR
    trap '_bash_base_err_exit_trap "EXIT"' EXIT
}

# bb_traps_disable
# disables traps for ERR and EXIT, must be called before clean shutdown
function bb_traps_disable() {
    trap - ERR
    trap - EXIT
}

# bb_version
# returns the version of bash base
function bb_version() {
    echo "0.2.4"
}

# bb_require_libs <name> [..]
# sources file for libs specified, it tries the following files
# /usr/libexec/bash-<name>.bash
# /usr/libexec/bash-<name>/bash-<name>.bash
# will fail if any lib fails to load
function bb_require_libs() {
    local l

    while (( $# > 0 ))
    do
	l="$1"
	shift

	if [ -f "/usr/libexec/bash-$l.bash" ]
	then
	    source "/usr/libexec/bash-$l.bash" || {
		fatal "bb_require_libs: " \
		    "error sourcing /usr/libexec/bash-$l.bash"
	    }
	    continue
	fi
	if [ -f "/usr/libexec/bash-$l/bash-$l.bash" ]
	then
	    source "/usr/libexec/bash-$l/bash-$l.bash" || {
		fatal "bb_require_libs: " \
		    "error sourcing /usr/libexec/bash-$l/bash-$l.bash"
	    }
	    continue
	else
	    fatal "bb_require_libs: no such lib found: $l"
	fi
    done
}

function _bash_base_main() {
    set -o errexit
    set -o pipefail
    set -o nounset
    set -o errtrace
}

_bash_base_main
